/* 
* File:   main.c
* Author: Jerome
*
* Created on November 21, 2013, 2:21 PM
*/

#define _CRT_SECURE_NO_WARNINGS

#include <stdio.h>
#include <stdlib.h>
//#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <windows.h>

#include "INOGENIHID.h"
#include "HIDCommand.h"

#define SOFT_VERSION "2.4"

typedef int (*cmd_call)(int argc, char * argv[]);

/* Command List */
int inogeni_cmd_help(int argc, char * argv[]);
int inogeni_cmd_Update_FPGA(int argc, char * argv[]);
int inogeni_cmd_Verify_FPGA(int argc, char * argv[]);
int inogeni_cmd_Update_FX3(int argc, char * argv[]);
int inogeni_cmd_Verify_FX3(int argc, char * argv[]);
int inogeni_cmd_Update_EDID(int argc, char * argv[]);
int inogeni_cmd_Verify_EDID(int argc, char * argv[]);
int inogeni_cmd_Get_Version(int argc, char * argv[]);
int inogeni_cmd_Get_Video(int argc, char * argv[]);
int inogeni_cmd_Reset(int argc, char * argv[]);
int inogeni_cmd_Set_Audio(int argc, char * argv[]);
int inogeni_cmd_Get_Descriptor(int argc, char * argv[]);
int inogeni_cmd_Set_Descriptor(int argc, char * argv[]);

struct inogeni_command {
	char *    cmd;
	char *    description;
	char *    usage;
	int       arg_count;
	cmd_call  callback;
};

struct inogeni_command commands[] = 
{  
	{
		"version",
		"Get version command",
		"",
		0,
		&inogeni_cmd_Get_Version
	},
	{
		"video",
		"Get video input info command",
		"",
		0,
		&inogeni_cmd_Get_Video
	},
	{
		"uedid",
		"Update EDID command",
		"<EDID bin file>",
		1,
		&inogeni_cmd_Update_EDID
	}, 
	{
		"vedid",
		"Verify EDID command",
		"<EDID bin file>",
		1,
		&inogeni_cmd_Verify_EDID
	},
	{
		"vfpga",
		"Verify FPGA EEPROM",
		"<FPGA bin file>",
		1,
		&inogeni_cmd_Verify_FPGA
	},
	{
		"ufpga",
		"Update FPGA EEPROM",
		"<FPGA bin file>",
		1,
		&inogeni_cmd_Update_FPGA
	},
	{
		"vfx3",
		"Verify FX3 EEPROM",
		 "<FX3 img file>",
		1,
		&inogeni_cmd_Verify_FX3
	},
	{
		"ufx3",
		"Update FX3 EEPROM",
		 "<FX3 img file>",
		1,
		&inogeni_cmd_Update_FX3
	},
	{
		"reset",
		"Reset FX3",
		"",
		0,
		&inogeni_cmd_Reset
	},
	{
		"setaudio",
		"Set Audio channel. Applies only on SHARE2 devices.\n" \
		"Possible parameter:\n" \
		"0 => AUTO\n" \
		"1 => HDMI\n" \
		"2 => DVI\n" \
		"3 => ANALOG\n",
		"<parameter>",
		1,
		&inogeni_cmd_Set_Audio
	},
	{
		"sdescr",
		"Set Descriptor",
		"",
		1,
		&inogeni_cmd_Set_Descriptor
	},
	{
		"gdescr",
		"Get Descriptor",
		"",
		0,
		&inogeni_cmd_Get_Descriptor
	},
	{
		"help",
		"Display help",
		"N/A",
		0,
		&inogeni_cmd_help
	},
	{
		0
	}
};

/*
* 
*/
int main(int argc, char** argv)
{
	//Decode command argument 

	if (argc <= 1)
	{
		printf("No command specified, use help command for list\n");
		return (EXIT_SUCCESS);
	}

	struct inogeni_command * current = commands;
	while (current->callback)
	{
		if (!strncmp(current->cmd,argv[1],3))
		{
			printf("Command: %s\n",current->description);
			if (argc < current->arg_count+2)
			{
				printf("Usage: %s\n",current->usage);
				return (EXIT_SUCCESS);
			}
			
			return (current->callback(argc,argv));
		}
		current++;
	}

	printf("Unrecognized Command\n");
	return (EXIT_SUCCESS);
}

int inogeni_cmd_help(int argc, char * argv[])
{
	struct inogeni_command * current = commands;
	printf ("INOGENICmdLineApp (Version: %s)\n",SOFT_VERSION);
	printf("Usages:\n");

	while (current->callback)
	{
		printf("Command:\t %s\n",current->description);
		printf("Usage:\t\t INOGENICmdLineApp %s %s\n\n",current->cmd,current->usage);
		current++;
	}

	return 0;
}

int inogeni_cmd_Reset(int argc, char * argv[])
{
	int result = Connect();
	if (result != 0)
	{
		printf("Error cannot connect\n");
		return -1;
	}
	
	FX3Reset();
	
	result = Disconnect();
	if (result != 0)
	{
		printf("Error in disconnecting\n");
		return -1;
	}
	printf("Resetting device.\n");
	printf("Please wait...\n");
	Sleep(7000);
	return 0;
}

int inogeni_cmd_Update_FPGA(int argc, char * argv[])
{
	char File[256];
	int result;
	
	// Only take a File Name in parameter
	if (argc > 3)
	{
		printf("Error: Too many parameters!\n");
		return 0;
	}
	else if (argc < 3)
	{
		printf("Error: Missing parameters!\n");
		return 0;
	}


	/* Get the FPGA EEPRM binary file name */
	sprintf (File,"%s" ,argv[argc-1] );

	result = Connect();
	if (result != 0)
	{
		printf("Error cannot connect\n");
		return -1;
	}

	enum ErrorCode Result = SPIFPGAProgram(File);
	if (Result == HID_ACK)
	{
		printf("FPGA Updated!\n"); 
	}
	
	result = Disconnect();
	if (result != 0)
	{
		printf("Error in disconnecting\n");
	}
	return 0;
}

int inogeni_cmd_Verify_FPGA(int argc, char * argv[])
{
	char File[256];
	int result;
	
	// Only take a File Name in parameter
	if (argc > 3)
	{
		printf("Error: Too many parameters!\n");
		return 0;
	}
	else if (argc < 3)
	{
		printf("Error: Missing parameters!\n");
		return 0;
	}


	/* Get the FPGA EEPRM binary file name */
	sprintf (File,"%s" ,argv[argc-1] );

	result = Connect();
	if (result != 0)
	{
		printf("Error cannot connect\n");
		return -1;
	}

	enum ErrorCode Result = SPIFPGAVerify(File);
	if (Result == HID_ACK)
	{
		printf("FPGA Verified!\n"); 
	}
	
	result = Disconnect();
	if (result != 0)
	{
		printf("Error in disconnecting\n");
	}
	return 0;
}

int inogeni_cmd_Update_FX3(int argc, char * argv[])
{
	char File[256];
	int result;
	
	// Only take a File Name in parameter
	if (argc > 3)
	{
		printf("Error: Too many parameters!\n");
		return 0;
	}
	else if (argc < 3)
	{
		printf("Error: Missing parameters!\n");
		return 0;
	}


	/* Get the FPGA EEPRM binary file name */
	sprintf (File,"%s" ,argv[argc-1] );

	result = Connect();
	if (result != 0)
	{
		printf("Error cannot connect\n");
		return -1;
	}

	enum ErrorCode Result = SPIFX3Program(File);
	if (Result == HID_ACK)
	{
		printf("FX3 Updated!\n"); 
	}
	
	result = Disconnect();
	if (result != 0)
	{
		printf("ERROR! error in disconnecting\n");
	}
	
	return 0;
}

int inogeni_cmd_Verify_FX3(int argc, char * argv[])
{
	char File[256];
	int result;
	
	// Only take a File Name in parameter
	if (argc > 3)
	{
		printf("Error: Too many parameters!\n");
		return 0;
	}
	else if (argc < 3)
	{
		printf("Error: Missing parameters!\n");
		return 0;
	}


	/* Get the FPGA EEPRM binary file name */
	sprintf (File,"%s" ,argv[argc-1] );

	result = Connect();
	if (result != 0)
	{
		printf("Error cannot connect\n");
		return -1;
	}

	enum ErrorCode Result = SPIFX3Verify(File);
	if (Result == HID_ACK)
	{
		printf("FX3 Verified!\n"); 
	}

	result = Disconnect();
	if (result != 0)
	{
		printf("Error in disconnecting\n");
	}
	return 0;
}

int inogeni_cmd_Update_EDID(int argc, char * argv[])
{
	char File[256];
	int result;
	
	// Only take a File Name in parameter
	if (argc > 3)
	{
		printf("Error: Too many parameters!\n");
		return 0;
	}
	else if (argc < 3)
	{
		printf("Error: Missing parameters!\n");
		return 0;
	}


	/* Get the FPGA EEPRM binary file name */
	sprintf (File,"%s" ,argv[argc-1] );


	result = Connect();
	if (result != 0)
	{
		printf("Error cannot connect\n");
		return -1;
	}

	enum ErrorCode Result = SPIEDIDProgram(File);
	if (Result == HID_ACK)
	{
		printf("EDID Updated!\n"); 
	}
	
	result = Disconnect();
	if (result != 0)
	{
		printf("ERROR! error in disconnecting\n");
	}
	
	return 0;
}

int inogeni_cmd_Verify_EDID(int argc, char * argv[])
{
	char File[256];
	int result;
	
	// Only take a File Name in parameter
	if (argc > 3)
	{
		printf("Error: Too many parameters!\n");
		return 0;
	}
	else if (argc < 3)
	{
		printf("Error: Missing parameters!\n");
		return 0;
	}


	/* Get the FPGA EEPRM binary file name */
	sprintf (File,"%s" ,argv[argc-1] );

	result = Connect();
	if (result != 0)
	{
		printf("Error cannot connect\n");
		return -1;
	}

	enum ErrorCode Result = SPIEDIDVerify(File);
	if (Result == HID_ACK)
	{
		printf("EDID Verified!\n"); 
	}
	
	result = Disconnect();
	if (result != 0)
	{
		printf("Error in disconnecting\n");
	}
	return 0;
}

int inogeni_cmd_Get_Version(int argc, char * argv[])
{
	int i = 0;
	int result = 0;
	int resultcode;
	unsigned char Value = 0;
	unsigned char Major = 0;
	unsigned char Minor = 0;

	
	result = Connect();
	if (result != 0)
	{
		printf("ERROR! error cannot connect\n");
		return -1;
	}
	
	
	resultcode = GetFx3Version(&Major, &Minor);
	if(resultcode == HID_ACK)
	{
		printf("FX3 version = %d.%d\n", Major, Minor);
	}
	else
	{
		printf("FX3 return error = %d\n", resultcode);
	}
	
	resultcode =  isFx3Initialize(&Value);
	if(resultcode == HID_ACK)
	{
		if(!Value)
		{
			printf("Device is not initialize cannot retrived EDID and FPGA version!\n");
			return -1;
		}
	}
	else
	{
		printf("FX3 return error = %d\n", resultcode);
	}
	
	
	
	resultcode = GetFPGAVersion(&Major, &Minor);
	if(resultcode == HID_ACK)
	{
		printf("FPGA version = %d.%d\n", Major, Minor);
	}
	else
	{
		printf("FX3 return error = %d\n", resultcode);
	}
	
	resultcode = GetEDIDVersion(&Major);
	if(resultcode == HID_ACK)
	{
		printf("EDID version = %d\n", Major);
	}
	else
	{
		printf("FX3 return error = %d\n", resultcode);
	}

	
	result = Disconnect();
	if (result != 0)
	{
		printf("ERROR! error in disconnecting\n");
	}
	

	return 0;
}

int inogeni_cmd_Get_Video(int argc, char * argv[])
{
	int result = 0;
	int resultcode;
	struct VideoFormat format;
	
	result = Connect();
	if (result != 0)
	{
		printf("ERROR! error cannot connect\n");
		return -1;
	}
	
	resultcode = GetInputVideoFormat(&format);
	if(resultcode == HID_ACK)
	{
		if(format.Locked)
		{
			printf("Video input = %dx%d %d.%d fps\n", format.ActivesPixel, format.ActiveLinesF0, format.FrameRate /100, format.FrameRate %100);
		}
		else
		{
			printf("There is no video input!\n");
		}
	}
	else
	{
		printf("FX3 return error = %d\n", resultcode);
	}
	
	
	result = Disconnect();
	if (result != 0)
	{
		printf("ERROR! error in disconnecting\n");
	}
	
	return 0;
}

int inogeni_cmd_Set_Audio(int argc, char * argv[])
{
	int result;
	unsigned int param = atoi(argv[2]);
	unsigned int value = 0;

	// Only take a File Name in parameter
	if (argc > 3)
	{
		printf("Error: Too many parameters!\n");
		return 0;
	}
	else if (argc < 3)
	{
		printf("Error: Missing parameters!\n");
		return 0;
	}

	result = Connect();
	if (result != 0)
	{
		printf("Error cannot connect\n");
		return -1;
	}

	if (param == 0)
		value = 0x0;
	else if (param == 1)
		value = 0x4;
	else if (param == 2)
		value = 0x5;
	else
		value = 0x6;

	enum ErrorCode Result = SetFX3Register(REG_AUDIO_SELECTOR, value);
	if (Result == HID_ACK)
	{
		printf("Audio channel set!\n");
	}

	result = Disconnect();
	if (result != 0)
	{
		printf("ERROR! error in disconnecting\n");
	}

	return 0;
}

int inogeni_cmd_Get_Descriptor(int argc, char * argv[])
{
	int result = 0;
	int resultcode;
	struct VideoFormat format;
	unsigned char data[INOGENI_REPORT_LEN];

	result = Connect();
	if (result != 0)
	{
		printf("ERROR! error cannot connect\n");
		return -1;
	}

	resultcode = ReadDescriptor(&data[0]);
	if (resultcode == HID_ACK)
	{
		printf("Descriptor = %s\n", data);
	}
	else
	{
		printf("FX3 return error = %d\n", resultcode);
	}


	result = Disconnect();
	if (result != 0)
	{
		printf("ERROR! error in disconnecting\n");
	}

	return 0;
}

int inogeni_cmd_Set_Descriptor(int argc, char * argv[])
{
	int result;
	unsigned int param = atoi(argv[2]);
	unsigned int value = 0;
	unsigned char data[INOGENI_REPORT_LEN];
	int resultcode;
	char descriptor[256];

	// Only take a File Name in parameter
	if (argc > 3)
	{
		printf("Error: Too many parameters!\n");
		return 0;
	}
	else if (argc < 3)
	{
		printf("Error: Missing parameters!\n");
		return 0;
	}

	result = Connect();
	if (result != 0)
	{
		printf("ERROR! error cannot connect\n");
		return -1;
	}

	/* Get the FPGA EEPRM binary file name */
	sprintf(descriptor, "%s", argv[argc - 1]);

	resultcode = WriteDescriptor((unsigned char*) &descriptor[0]);
	if (resultcode == HID_ACK)
	{
		printf("Descriptor = %s\n", data);
	}
	else
	{
		printf("FX3 return error = %d\n", resultcode);
	}


	result = Disconnect();
	if (result != 0)
	{
		printf("ERROR! error in disconnecting\n");
	}

	return 0;
}


